package xyz.lychee.lagfixer.libs.adventure.translation;

import java.text.MessageFormat;
import java.util.Collections;
import java.util.Iterator;
import java.util.Locale;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.key.Key;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.TranslatableComponent;
import xyz.lychee.lagfixer.libs.adventure.text.renderer.TranslatableComponentRenderer;
import xyz.lychee.lagfixer.libs.adventure.util.TriState;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

final class GlobalTranslatorImpl implements GlobalTranslator {
   private static final Key NAME = Key.key("adventure", "global");
   static final GlobalTranslatorImpl INSTANCE = new GlobalTranslatorImpl();
   final TranslatableComponentRenderer<Locale> renderer = TranslatableComponentRenderer.usingTranslationSource(this);
   private final Set<Translator> sources = Collections.newSetFromMap(new ConcurrentHashMap());

   private GlobalTranslatorImpl() {
   }

   public @NotNull Key name() {
      return NAME;
   }

   public @NotNull Iterable<? extends Translator> sources() {
      return Collections.unmodifiableSet(this.sources);
   }

   public boolean addSource(final @NotNull Translator source) {
      Objects.requireNonNull(source, "source");
      if (source == this) {
         throw new IllegalArgumentException("GlobalTranslationSource");
      } else {
         return this.sources.add(source);
      }
   }

   public boolean removeSource(final @NotNull Translator source) {
      Objects.requireNonNull(source, "source");
      return this.sources.remove(source);
   }

   public @NotNull TriState hasAnyTranslations() {
      return !this.sources.isEmpty() ? TriState.TRUE : TriState.FALSE;
   }

   public @Nullable MessageFormat translate(final @NotNull String key, final @NotNull Locale locale) {
      Objects.requireNonNull(key, "key");
      Objects.requireNonNull(locale, "locale");
      Iterator var3 = this.sources.iterator();

      MessageFormat translation;
      do {
         if (!var3.hasNext()) {
            return null;
         }

         Translator source = (Translator)var3.next();
         translation = source.translate(key, locale);
      } while(translation == null);

      return translation;
   }

   public @Nullable Component translate(final @NotNull TranslatableComponent component, final @NotNull Locale locale) {
      Objects.requireNonNull(component, "component");
      Objects.requireNonNull(locale, "locale");
      Iterator var3 = this.sources.iterator();

      Component translation;
      do {
         if (!var3.hasNext()) {
            return null;
         }

         Translator source = (Translator)var3.next();
         translation = source.translate(component, locale);
      } while(translation == null);

      return translation;
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("sources", this.sources));
   }
}
